﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.IO;
using System.Windows.Forms;

namespace Musiclibrary
{
    /// <summary>
    /// A popup showing the library's settings.
    /// </summary>
    public partial class Settings : Popup
    {
        public Settings()
        {
            InitializeComponent();

            //Load settings
            this.OrganizeLibrary.IsChecked = App.This.Library.Organize;

            if (App.This.Library.Organize)
            {
                this.Directory.Text = App.This.Library.Location;

                var schema = App.This.Library.Schema;

                if (schema == this.Schema1.Tag.ToString())
                    this.Schema1.IsChecked = true;
                else if (schema == this.Schema2.Tag.ToString())
                    this.Schema2.IsChecked = true;
                else
                {
                    this.CustomSchema.IsChecked = true;
                    this.Schema.Text = schema;
                }
            }

            foreach (var dir in App.This.Library.MonitoredDirs)
                this.ObservedFolders.Items.Add(dir);
        }

        private void Browse_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new FolderBrowserDialog();
            dialog.Description = "Choose the folder where you want to locate your music:";
            dialog.ShowNewFolderButton = true;

            if (dialog.ShowDialog() == DialogResult.OK)
                this.Directory.Text = dialog.SelectedPath;
        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            //If the user wants to organize his library, he's got to say where!
            if (this.OrganizeLibrary.IsChecked.Value && !System.IO.Directory.Exists(this.Directory.Text))
            {
                Extensions.ShowErrorBox("You have to specify a location for the library if you want your library to be automatically organized!");
                return;
            }

            //Replace observed directories
            App.This.Library.MonitoredDirs.DeleteAllOnSubmit(App.This.Library.MonitoredDirs);
            App.This.Library.MonitoredDirs.InsertAllOnSubmit(this.ObservedFolders.Items.Cast<MonitoredDir>());

            string schema;
            if (this.Schema1.IsChecked.Value)
                schema = this.Schema1.Tag.ToString();
            else if (this.Schema2.IsChecked.Value)
                schema = this.Schema2.Tag.ToString();
            else
                schema = this.Schema.Text;

            //Determine whether the library has to be reorganized
            var reorganize = this.OrganizeLibrary.IsChecked.Value &&
                (schema != App.This.Library.Schema ||
                 this.Directory.Text != App.This.Library.Location);

            //Determine whtether the library has to be organized and was not organized before
            var newOrganize = !App.This.Library.Organize && this.OrganizeLibrary.IsChecked.Value;

            App.This.Library.Organize = this.OrganizeLibrary.IsChecked.Value;

            if (this.OrganizeLibrary.IsChecked.Value)
            {
                App.This.Library.Schema = schema;
                App.This.Library.Location = this.Directory.Text;
            }
            App.This.Library.SubmitChanges();

            //Reorganize library
            if (reorganize) {
                var reorganizer = new Reorganizer(App.This.Library, this);
                reorganizer.Move = true;

                //If library is organized first time, ask user if he wants to move or copy his files
                if (newOrganize && App.This.Library.Songs.Count() > 0 &&
                        System.Windows.MessageBox.Show("Your library was not organized until now. Do you want to move the files from their current place to the specified place? Click \"No\" to copy them instead of moving.",
                        "Musiclibrary", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.No)
                    reorganizer.Move = false;

                this.Adorner.ShowNextDialog(new Working(reorganizer, "Reorganizing...", "Your music library is being re-organized"), false);

                reorganizer.Done += new EventHandler<WorkerCompletedEventArgs>((sender2, e2) => {
                    this.Adorner.Hide();
                    App.This.Watcher.LoadDirectories();
                });
                reorganizer.Cancelled += new EventHandler<EventArgs>((sender2, e2) =>
                    this.Adorner.Hide());

                reorganizer.Begin();
            } else {
                this.Adorner.Hide();
                App.This.Watcher.LoadDirectories();
            }
        }

        private void ObservedFolders_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.RemoveFolder.IsEnabled = this.ObservedFolders.SelectedItems.Count > 0;
        }

        private void RemoveFolder_Click(object sender, RoutedEventArgs e)
        {
            while (this.ObservedFolders.SelectedItems.Count > 0)
                this.ObservedFolders.Items.Remove(this.ObservedFolders.SelectedItems[0]);
        }

        private void AddFolder_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new System.Windows.Forms.FolderBrowserDialog();
            dialog.Description = "Choose a folder whose files should be automatically added to the library.";

            if (dialog.ShowDialog() == DialogResult.OK)
                this.ObservedFolders.Items.Add(new MonitoredDir()
                {
                    Path = dialog.SelectedPath
                });
        }
    }
}
