﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Musiclibrary {
    /// <summary>
    /// Provides an abstraction for saving metadata in a music file
    /// and a function to read the metadata from a file.
    /// </summary>
    public abstract class MetadataBase {
        /// <summary>
        /// Loads the metadata from the specified file.
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        public static MetadataBase LoadMetadata(string file) {
            var info = new UltraID3Metadata();
            info.Load(file);
            return info;
        }

        public abstract string Title { get; set; }
        public abstract string Artist { get; set; }
        public abstract string Album { get; set; }
        public abstract string Genre { get; set; }
        public abstract int? Year { get; set; }
        public abstract string Path { get;  }

        /// <summary>
        /// Writes the metadata back to the file.
        /// </summary>
        public abstract void Save();

        /// <summary>
        /// Returns a new Song object with the metadata of this object.
        /// </summary>
        /// <returns></returns>
        public Song GetSong() {
            var song = new Song();
            song.Title = this.Title;
            song.Artist = this.Artist;
            song.Album = this.Album;
            song.Genre = this.Genre;
            song.Year = this.Year;
            song.Path = this.Path;
            return song;
        }
    }
}
