﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.ComponentModel;
using System.Threading;
using System.Data.SqlServerCe;

namespace Musiclibrary
{
    public partial class Library : ILibrary
    {
        /// <summary>
        /// Creates a new library. Do not use CreateDatabase!
        /// </summary>
        public static void Create(string path)
        {
            //Create database file
            using (var engine = new SqlCeEngine("Data Source=" + path))
                engine.CreateDatabase();

            //Open connection
            var conn = new SqlCeConnection("Data Source=" + path);
            conn.Open();

            //Execute sql commands to create database
            foreach (var command in Properties.Resources.SQL.Split(new char[] { ';' }))
            {
                var cmd = new SqlCeCommand(command, conn);
                cmd.ExecuteNonQuery();
            }

            //Close connection
            conn.Close();
            conn.Dispose();
        }

        public string Name {
            get { return Environment.UserName + "s library"; }
        }

        #region Settings

        /// <summary>
        /// Determines whether the library should be organized (nice folder structure).
        /// </summary>
        public bool Organize {
            get {
                var setting = (from i in this.Settings where i.SKey == "organize" select i);
                var value = setting.Single().SValue;
                return value == "1";
            }
            set {
                //Another SQL CE weirdness: This query wasn't possible using LINQ to SQL..
                this.ExecuteQuery<object>("UPDATE settings SET sValue = {0} WHERE sKey = {1}", value ? "1" : "0", "organize");
            }
        }

        /// <summary>
        /// Determines the path where the library should be organized.
        /// </summary>
        public string Location {
            get {
                return (from i in this.Settings where i.SKey == "location" select i.SValue).Single();
            }
            set {
                this.ExecuteQuery<object>("UPDATE settings SET sValue = {0} WHERE sKey = {1}", value, "location");
            }
        }

        /// <summary>
        /// Determines the schema how the library should be organized.
        /// </summary>
        public string Schema {
            get {
                return (from i in this.Settings where i.SKey == "schema" select i.SValue).Single();
            }
            set {
                this.ExecuteQuery<object>("UPDATE settings SET sValue = {0} WHERE sKey = {1}", value, "schema");
            }
        }

        #endregion


        #region ILibrary Member

        IEnumerable<ISong> ILibrary.Songs {
            get { return this.Songs.Cast<ISong>(); }
        }

        IEnumerable<IPlaylist> ILibrary.Playlists {
            get { return this.Playlists.Cast<IPlaylist>(); }
        }

        #endregion
    }
}
