﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Musiclibrary.Properties;
using System.ComponentModel;
using Microsoft.WindowsAPICodePack.Taskbar;
using System.Reflection;
using System.Windows.Interop;
using System.Windows;

namespace Musiclibrary
{
    /// <summary>
    /// Provides functionality for Win7 integration like jump lists and thumbnail toolbars.
    /// </summary>
    public class Win7Tools
    {
        protected JumpList JumpList;

        public Win7Tools(Window window)
        {
            if (!TaskbarManager.IsPlatformSupported) return;

            //Create thumbnail toolbar
            var previous = new ThumbnailToolbarButton(Properties.Resources.previous, "Previous song");
            var playPause = new ThumbnailToolbarButton(Properties.Resources.play, "Play");
            var next = new ThumbnailToolbarButton(Properties.Resources.next, "Next song");

            previous.Enabled = false;
            next.Enabled = false;

            //Dynamically enable/disable previous/next button
            App.This.Player.StateChanged += new Action<IPlayer, EventArgs>((sender, e) => {
                if (App.This.Player.State == PlayerState.Playing) {
                    playPause.Icon = Properties.Resources.pause;
                    playPause.Tooltip = "Pause";
                } else {
                    playPause.Icon = Properties.Resources.play;
                    playPause.Tooltip = "Play";
                }
            });

            App.This.Player.SongIndexChanged += new Action<IPlayer, EventArgs>((sender, e) => {
                previous.Enabled = App.This.Player.SongIndex > 0 || App.This.Player.Repeat;
                next.Enabled = App.This.Player.SongIndex < App.This.Player.Playlist.Count - 1 || App.This.Player.Repeat;
            });

            previous.Click += new EventHandler<ThumbnailButtonClickedEventArgs>((sender,e) => 
                App.This.Player.Previous());

            next.Click += new EventHandler<ThumbnailButtonClickedEventArgs>((sender, e) =>
                App.This.Player.Next());

            playPause.Click += new EventHandler<ThumbnailButtonClickedEventArgs>((sender, e) =>
                MainWindow.Instance.PlayOrPause());

            TaskbarManager.Instance.ThumbnailToolbars.AddButtons(
                new WindowInteropHelper(window).Handle,
                previous, playPause, next);

            this.JumpList = JumpList.CreateJumpList();
        }


        /// <summary>
        /// Loads the playlists into the jump list.
        /// </summary>
        public void LoadPlaylists()
        {
            if (!TaskbarManager.IsPlatformSupported) return;

            var category = new JumpListCustomCategory("Playlists");

            foreach (var playlist in App.This.Library.Playlists)
            {
                var item = new JumpListLink(Assembly.GetExecutingAssembly().Location, playlist.Name);
                item.Arguments = "-playlist " + playlist.Id.ToString();
                
                category.AddJumpListItems(item);
            }

            this.JumpList.AddCustomCategories(category);
            this.JumpList.Refresh();
        }

        //Sets the current progress state displayed in the taskbar icon
        public static void SetProgressState(TaskbarProgressBarState state)
        {
            if (TaskbarManager.IsPlatformSupported)
                TaskbarManager.Instance.SetProgressState(state);
        }

        //Sets the current progress if ProgressState is set to Normal
        public static void SetProgress(double value)
        {
            if (TaskbarManager.IsPlatformSupported)
                TaskbarManager.Instance.SetProgressValue((int)(value * 100), 100);
        }
    }
}
