﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Threading;
using System.IO;

namespace Musiclibrary
{
    /// <summary>
    /// A Worker that reorganizes the library after the user has changed the organization scheme
    /// or location.
    /// </summary>
    public class Reorganizer : Worker
    {
        public Reorganizer(Library library, DispatcherObject invokeTarget)
            : base(invokeTarget)
        {
            this.Library = library;
        }

        protected Library Library { get; set; }

        public bool Move { get; set; }

        protected override object DoWork()
        {
            var failed = new Dictionary<Song, Exception>();

            try {
                var count = this.Library.Songs.Count();
                var i = 1;

                foreach (var song in this.Library.Songs)
                {
                    this.CurrentState = "Moving \"" + song.Title + "\"...";

                    var newPath = song.GetPath(this.Library.Location, this.Library.Schema, true);
                    var newDir = Path.GetDirectoryName(newPath);

                    try
                    {
                        if (!Directory.Exists(newDir))
                            Directory.CreateDirectory(newDir);

                        if (this.Move)
                            File.Move(song.Path, newPath);
                        else
                            File.Copy(song.Path, newPath);

                        song.Path = newPath;
                    }
                    catch (Exception ex)
                    {
                        failed.Add(song, ex);
                    }
                    
                    this.Progress = ((double)i++ / (double)count);

                    if (this.CancelPending) return null;
                }

                App.This.Library.SubmitChanges();
            } 
            catch (Exception ex) {
                failed.Clear();
                failed.Add(new Song() { Title = "General error" }, ex);
            }

            return failed;
        }
    }
}
